#!/bin/sh
CC=gcc
debug=no
log=unknown
check=unknown
visualize=no
forward=undefined
staticvisited=undefined
LC=0
LV=0
usage () {
cat <<EOF
usage: ./configure.sh [ <option> ... ]

where <option> is one of the following

-h print this command line option summary

-g compile in debugging mode ('-g' and no '-O3')
-l include logging code even in non-debugging mode
-c enable assertion checking also in non-debugging mode
-s static allocation for visited (forces '-f')
-f forward checking only: disables hybrid and backward only checking

LC=<BITS>   maximum coordinate bits (default 9, BITS=6..12)
LV=<BITS>   maximum velocity bits (default 7, BITS=4..10)

-v include code for visualization
   (needs OpenGL, e.g., mesa-common-dev, freeglut3-dev on Ubuntu)
EOF
}
die () {
  echo "*** configure.sh: $*" 1>&2
  exit 1
}
while [ $# -gt 0 ]
do
  case $1 in
    -h) usage; exit 1;;
    -g) debug=yes;;
    -l) log=yes;;
    -c) check=yes;;
    -1) onefile=yes;;
    -s) staticvisited=yes;;
    -v) visualize=yes;;
    -f) forward=yes;;
    "LC=6") LC=6;;
    "LC=7") LC=7;;
    "LC=8") LC=8;;
    "LC=9") LC=9;;
    "LC=10") LC=10;;
    "LC=11") LC=11;;
    "LC=12") LC=12;;
    "LV=4") LV=4;;
    "LV=5") LV=5;;
    "LV=6") LV=6;;
    "LV=7") LV=7;;
    "LV=8") LV=8;;
    "LV=9") LV=9;;
    "LV=10") LV=10;;
    *) die "invalid option '$1' (try '-h')";;
  esac
  shift
done
if [ $staticvisited = undefined ]
then
  if [ $forward = yes ]
  then
    staticvisited=yes
  else
    staticvisited=no
  fi
fi
if [ $forward = undefined ]
then
  if [ $staticvisited = yes ]
  then
    forward=yes
  else
    forward=no
  fi
fi
if [ $staticvisited = yes ]
then
  [ $LC -gt 9 ] && die "LC=$LC too large for '-s'"
  [ $LV -gt 7 ] && die "LV=$LV too large for '-s'"
elif [ $forward = no ]
then
  staticvisited=no
elif [ $LC = 0 -a $LV = 0 ]
then
  staticvisited=yes
fi
if [ $visualize = yes ]
then
  [ -d /usr/include/GL ] || \
    die "can not find directory '/usr/include/GL' (needed for '-v')"
  [ -f /usr/include/GL/gl.h ] || \
    die "can not find file '/usr/include/GL/gl.h' (needed for '-v')"
  [ -f /usr/include/GL/glut.h ] || \
    die "can not find file '/usr/include/GL/glut.h' (needed for '-v')"
fi
CFLAGS="-Wall"
if [ $debug = yes ]
then
  [ $log = yes ] && \
    die "'-l' does not make sense in combination with '-g'"
  [ $check = yes ] && \
    die "'-c' does not make sense in combination with '-g'"
  CFLAGS="$CFLAGS -g3"
else
  CFLAGS="$CFLAGS -O3"
fi
[ $check = unknown ] && check=$debug
[ $log = unknown ] && log=$debug
[ $check = no ] && CFLAGS="$CFLAGS -DNDEBUG"
[ $log = no ] && CFLAGS="$CFLAGS -DNLOG"
OBJ="mcracetrack.o common.o config.o"
DEP="mcracetrack.c search.c common.h state.h"
LIB=""
if [ $visualize = yes ]
then
 CFLAGS="$CFLAGS -DVISUALIZE"
 LIB="$LIB -lGLU -lglut -lGL -pthread"
 OBJ="$OBJ visualize.o"
 DEP="$DEP visualize.h"
fi
[ $staticvisited = yes ] && CFLAGS="$CFLAGS -DSTATICVISITED"
[ $forward = no ] && CFLAGS="$CFLAGS -DBACKWARD"
[ $LC = 0 ] || CFLAGS="$CFLAGS -DLC=$LC"
[ $LV = 0 ] || CFLAGS="$CFLAGS -DLV=$LV"
echo "$CC $CFLAGS"
sed \
-e "s,@CC@,$CC," \
-e "s,@CFLAGS@,$CFLAGS," \
-e "s,@OBJ@,$OBJ," \
-e "s,@DEP@,$DEP," \
-e "s,@LIB@,$LIB," \
makefile.in > makefile
